import datetime
from typing import Union, Type

DATETIME_CLS = datetime.datetime
DATE_CLS = datetime.date
TIME_CLS = datetime.time
SUPPORTED_TIME_CLASSES = Union[DATETIME_CLS, DATE_CLS, TIME_CLS]
SUPPORTED_TIME_CLASSES_TYPES = Union[Type[DATE_CLS], Type[DATETIME_CLS], Type[TIME_CLS]]
TYPE_KEY = '__type__'
VALUE_KEY = '__value__'



import json
from typing import Dict, Optional

from json_datetime_noamfisher.consts import (VALUE_KEY, TYPE_KEY, SUPPORTED_TIME_CLASSES, SUPPORTED_TIME_CLASSES_TYPES)


class JsonDatetimeDecoder(json.JSONDecoder):
    def __init__(self, *args, **kwargs):
        super(JsonDatetimeDecoder, self).__init__(*args, object_hook=self._object_hook, **kwargs)

    def _object_hook(self, obj: Dict):
        obj_type = obj.get(TYPE_KEY)
        value = obj.get(VALUE_KEY)

        obj_cls = self._get_obj_datetime_cls(obj_type)
        if obj_cls is not None and isinstance(value, str) and len(obj) == 2:
            try:
                return obj_cls.fromisoformat(value)
            except ValueError:
                pass

        return obj

    def _get_obj_datetime_cls(self, obj_type: str) -> Optional[SUPPORTED_TIME_CLASSES_TYPES]:
        for cls in SUPPORTED_TIME_CLASSES.__args__:
            if cls.__name__ == obj_type:
                return cls

    def _dead_code(self):
        print(123)


import datetime
import json
import os

import pytest

from json_datetime_noamfisher.consts import DATETIME_CLS, DATE_CLS, TIME_CLS
from json_datetime_noamfisher.decoder import JsonDatetimeDecoder
from json_datetime_noamfisher.encoder import JsonDatetimeEncoder


@pytest.mark.parametrize(
    'the_json',
    [
        {'a': DATETIME_CLS(2020, 3, 24, 11, 52, 34, 123456)},
        {'a': DATE_CLS(2020, 3, 24)},
        {'a': TIME_CLS(12, 34, 56, 789012)},
        [DATETIME_CLS(2020, 3, 24, 11, 52, 34, 123456), DATE_CLS(2020, 3, 24), TIME_CLS(12, 34, 56, 789012)],
        {'a': [DATETIME_CLS(2020, 3, 24, 11, 52, 34, 123456), DATE_CLS(2020, 3, 24), TIME_CLS(12, 34, 56, 789012)]}
    ])
def test_encode_decode(the_json):
    encoded = json.dumps(the_json, cls=JsonDatetimeEncoder)
    decoded = json.loads(encoded, cls=JsonDatetimeDecoder)
    assert the_json == decoded


class A:
    pass


@pytest.mark.parametrize(
    'the_json',
    [
        {'a': A()}
    ]
)
def test_not_serializable_object(the_json):
    with pytest.raises(TypeError):
        json.dumps(the_json, cls=JsonDatetimeEncoder)


@pytest.mark.parametrize(
    'encoded, expected_decoded',
    [
        (
                """{"a": {"__type__": "datetime", "__value__": "not_a_valid_datetime_value"}}""",
                {"a": {"__type__": "datetime", "__value__": "not_a_valid_datetime_value"}}
        )
    ]
)
def test_not_deserializable_json(encoded: str, expected_decoded):
    decoded = json.loads(encoded, cls=JsonDatetimeDecoder)
    assert decoded == expected_decoded


@pytest.mark.skipif(os.environ.get('COVID_19_DATA_PATH') is None,
                    reason="missing COVID_19_DATA_PATH environment variable")
def test_data_file():
    path = os.environ['COVID_19_DATA_PATH']
    patients = json.load(open(path), cls=JsonDatetimeDecoder)
    for patient in patients:
        assert isinstance(patient['infection_time'], datetime.datetime)



import json
from typing import Union, Any

from json_datetime_noamfisher.consts import (SUPPORTED_TIME_CLASSES, VALUE_KEY, TYPE_KEY)


class JsonDatetimeEncoder(json.JSONEncoder):
    def default(self, o: Union[Any, SUPPORTED_TIME_CLASSES]):
        if isinstance(o, SUPPORTED_TIME_CLASSES.__args__):
            return {
                TYPE_KEY: type(o).__name__,
                VALUE_KEY: o.isoformat()
            }
        return super(JsonDatetimeEncoder, self).default(o)



import os
from typing import Optional

import setuptools


def get_version() -> Optional[str]:
    version_file_path = './version'
    if os.path.isfile('./version'):
        with open(version_file_path, 'r') as version_file:
            return version_file.read()


setuptools.setup(
    name=f'json-datetime-noamfisher',
    version=get_version(),
    description='allows you to dump / load jsons that contain date / datetime / time objects',
    author=os.environ.get('GITLAB_USER_NAME'),
    author_email=os.environ.get('GITLAB_USER_EMAIL'),
    extras_require={
        'test': ['pytest']
    },
    packages=setuptools.find_packages()
)
